Unit ScreenGrab;
{
  ==================================================================
                     Copyright 1999 by Patrick Spence
                            All rights reserved
               Some portions copyright 1999 by Daniel Stasinski

     You may freely distribute this code as long as you leave my
     copyright notice intact
  ==================================================================

  Documentation is minimal in this release
  If you have any questions, feel free to email me at:
  tachyon@cryptic.org

  GetFullScreen  grabs the current full screen
  GetBMP         grabs the rectangle specified in the StartX, StartY
                 Width and Height properties
  FileName       Is the name saved to when the SaveAsBMP procedure is run

  1999-11-18
}
Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

Type
  tScreenGrab = Class(TComponent)
  Private
    Bmp: tBitMap;
    LeftSide: word;
    Top: word;
    TheWidth: Word;
    TheHeight: Word;
    filname: String;
    { Private declarations }
    Procedure SnapShot(xpos: integer; ypos: integer; Wid: integer; Height: integer; Var bmap: TBitMap);
    Function GetStartX: word;
    Function GetStartY: word;
    Function GetWidth: word;
    Function GetHeight: word;
    Function GetFileName: String;
    Function GetTheBMP: tBitMap;
    Procedure SetStartX(w: word);
    Procedure SetStartY(w: word);
    Procedure Setwidth(w: word);
    Procedure Setheight(w: word);
    Procedure SetFileName(s: String);
    Procedure SetTheBMP(B: tBitMap);
  Protected
    { Protected declarations }
    Constructor Create(AOwner: TComponent);
  Public
    { Public declarations }
    Procedure UnixName; // Gets a Unix timestamp style name for BMP
    Procedure GetFullScreen; // Gets just a screen
    Procedure GetBMP; // Gets specified Rect of screen
    Procedure SaveAsBMP; // Saves as bitmap to current name
  Published
    { Published declarations }
    Property StartX: word Read GetStartX Write SetStartX;
    Property StartY: word Read GetStartY Write SetStartY;
    Property width: word Read GetWidth Write SetWidth;
    Property height: word Read GetHeight Write SetHeight;
    Property FileName: String Read GetFileName Write SetFileName;
    Property TheBMP: tBitMap Read GetTheBMP Write SetTheBMP;
  End;

Procedure Register;
Function DelphiDateTimeToUnix(ConvDate: TdateTime): longint;
          // Converts Delphi date/time to Unix time stamp
Function UnixToDelphiDateTime(USec: longint): TDateTime;
          // Converts Unix Timestamp to Delphi date/time
Function UnixNow: LongInt;
          // Returns current unix timestamp
Function Rpt(S: String; Rp: Byte): String;
          // Returns S repeated RP times
Function PadNum(Const S: String; Size: Byte): String;
          // Pads '0' onto beginning of S, to Size length
Function HasExtension(Fname: String; Var DotPos: Byte): Boolean;
          // checks to see if has extension
Function DefaultExtension(FileName, Ext: String): String;
          // Puts a default extension on file

Implementation
Const
  UnixStartDate     : tdatetime = 25568.0;  // for unix routines

Procedure Register;
Begin
  RegisterComponents('EIS', [tScreenGrab]);
End;

Function DefaultExtension(FileName, Ext: String): String;
Var
  DotPos            : Byte;
Begin
  If HasExtension(FileName, DotPos) Then
    DefaultExtension := FileName
  Else
    DefaultExtension := FileName + Ext;
End;

Function HasExtension(Fname: String; Var DotPos: Byte): Boolean;
Var
  Loop              : Word;
Begin
  DotPos := 0;
  For Loop := Length(FName) Downto 1 Do
    If (FName[Loop] = '.') And (DotPos = 0) Then
      DotPos := Loop;
  HasExtension := (DotPos > 0) And (Pos('\', Copy(Fname, Succ(DotPos), 64)) =
    0);
End;



Function Rpt(S: String; Rp: Byte): String;
Var
  TStr              : String;
  B                 : Byte;
Begin
  TStr := '';
  If (Rp > 0) Then
    For B := 1 To Rp Do
      TStr := TStr + S;
  Rpt := TStr;
End;

Function PadNum(Const S: String; Size: Byte): String;
Begin
  If (Length(S) < Size) Then
    PadNum := (Rpt('0', (Size - Length(S))) + S)
  Else
    PadNum := S;
End;

Function DelphiDateTimeToUnix(ConvDate: TdateTime): longint;
Begin
  Try
    Result := round((ConvDate - UnixStartDate) * 86400);
  Except
    Result := 0;
  End;
End;

Function UnixToDelphiDateTime(USec: longint): TDateTime;
Begin
  Try
    Result := (Usec / 86400) + UnixStartDate;
  Except
    Result := 0;
  End;
End;

Function UnixNow: LongInt;
Begin
  UnixNow := DelphiDateTimeToUnix(Now);
End;


Constructor tScreenGrab.Create(AOwner: TComponent);
Begin
  Inherited Create(aowner);
  BMP := Nil;
  width := screen.desktopwidth;
  height := screen.DesktopHeight;
  StartX := screen.DesktopLeft;
  StartY := Screen.Desktopheight;
  FilName := '';
End;

Function tScreenGrab.GetStartX: word;
Begin
  result := leftside;
End;

Function tScreenGrab.GetStartY: word;
Begin
  result := top;
End;

Function tScreenGrab.GetWidth: word;
Begin
  result := TheWidth;
End;

Function tScreenGrab.GetHeight: word;
Begin
  result := TheHeight;
End;

Procedure tScreenGrab.SetStartX(w: word);
Begin
  LeftSide := w;
End;

Procedure tScreenGrab.SetStartY(w: word);
Begin
  Top := W;
End;

Procedure tScreenGrab.Setwidth(w: word);
Begin
  TheWidth := w;
End;

Procedure tScreenGrab.Setheight(w: word);
Begin
  TheHeight := w;
End;

Procedure tScreenGrab.UnixName;
Begin
  FileName := DefaultExtension(PadNum(IntToStr(UnixNow), 8), '.bmp');
End;

Function tScreenGrab.GetTheBMP: tBitMap;
Begin
  Result := bmp;
End;

Procedure tScreenGrab.SetTheBMP(B: tBitMap);
Begin
  bmp := B;
End;

Function tScreenGrab.GetFileName: String;
Begin
  Result := Filname;
End;

Procedure tScreenGrab.SetFileName(s: String);
Begin
  FilName := S;
End;

Procedure tScreenGrab.SnapShot(xpos: integer; ypos: integer; Wid: integer; Height: integer; Var bmap:
  TBitMap);
Var
  dc                : HDC;
  lpPal             : PLOGPALETTE;
Begin
  If ((Wid = 0) Or (Height = 0)) Then
    exit;
  bmap.Width := Wid;
  bmap.Height := Height;
  dc := GetDc(0);
  If (dc = 0) Then
    exit;

  If (GetDeviceCaps(dc, RASTERCAPS) And
    RC_PALETTE = RC_PALETTE) Then
  Begin
    GetMem(lpPal, sizeof(TLOGPALETTE) + (255 * sizeof(TPALETTEENTRY)));
    FillChar(lpPal^, sizeof(TLOGPALETTE) + (255 * sizeof(TPALETTEENTRY)), #0);
    lpPal^.palVersion := $300;
    lpPal^.palNumEntries := GetSystemPaletteEntries(dc, 0, 256, lpPal^.palPalEntry);
    If (lpPal^.PalNumEntries <> 0) Then
      bmap.Palette := CreatePalette(lpPal^);
    FreeMem(lpPal, sizeof(TLOGPALETTE) + (255 * sizeof(TPALETTEENTRY)));
  End;
  BitBlt(bmap.Canvas.Handle, 0, 0, Wid, Height, Dc, xpos, ypos, SRCCOPY);
  ReleaseDc(0, dc);
End;

Procedure tScreenGrab.GetBMP;
Begin
  Bmp := tBitMap.create;
  Try
    SnapShot(LeftSide, Top, TheWidth, Height, Bmp);
  Except
  End;
End;

Procedure tScreenGrab.GetFullScreen; // Gets just a screen
Begin
  Bmp := tBitMap.create;
  Try
    SnapShot(screen.desktopLeft, Screen.DeskTopTop, Screen.DeskTopWidth, Screen.DeskTopHeight, Bmp);
  Except
  End;
End;

Procedure tScreenGrab.SaveAsBMP; // Saves as bitmap to current name
Begin
  If FilName <> '' Then
  Try
    Bmp.SaveToFile(FilName);
  Except
  End;
End;


End.

